/* $Id: compat.h,v 1.2 2007-01-24 12:11:52 kristaps Exp $ */
/*
Copyright (c) 2006 Kristaps Dzonsons
Latvijas Universitaates Matemaatikas un Informaatikas Instituuts
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:
* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
* Neither the name of the copyright holders nor the names of
  contributors may be used to endorse or promote products derived from
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef PL_COMPAT_H
#define PL_COMPAT_H

__BEGIN_DECLS

#if ! defined(__GNUC__) || ! defined(__attribute__)
# define __attribute__(x) /*Nothing*/
#endif

/* Account for BSD stuff on Linux machines.  We should use
 * -D_BSD_SOURCE, but I'd like to keep the code as clean from
 *  compatibility issues.  The timer helpers are fairly
 *  straight-forward, but rand() must be replaced. */

#ifndef __linux__
   /* Assume that these have strotnum and setproctitle. */
#  define HAS_STRTONUM
#  define HAS_SETPROCTITLE
#else /*!__linux__*/
   /* Linux has these but they're protected with _BSD_SOURCE. */
extern long long
strtonum(const char *numstr, long long minval, long long maxval,
	const char **errstrp);

extern void
setproctitle(const char *fmt, ...) 
	__attribute__ ((format (printf, 1, 2)));

extern void
srandom(unsigned int seed);

extern long int
random(void);

#define timersub(a, b, result)			      	      \
  do {							      \
    (result)->tv_sec = (a)->tv_sec - (b)->tv_sec;	      \
    (result)->tv_usec = (a)->tv_usec - (b)->tv_usec;	      \
    if ((result)->tv_usec < 0) {			      \
      --(result)->tv_sec;				      \
      (result)->tv_usec += 1000000;			      \
    }							      \
  } while (0)

#define timeradd(a, b, result)		 	      	      \
  do {							      \
    (result)->tv_sec = (a)->tv_sec + (b)->tv_sec;	      \
    (result)->tv_usec = (a)->tv_usec + (b)->tv_usec;	      \
    if ((result)->tv_usec >= 1000000)			      \
      {							      \
	++(result)->tv_sec;				      \
	(result)->tv_usec -= 1000000;			      \
      }							      \
  } while (0)

#endif /*!__linux__*/

__END_DECLS

#endif /*!PL_COMPAT_H*/
