/* $Id: main.c,v 1.25 2007-01-22 21:38:18 kristaps Exp $ */
/*
Copyright (c) 2006 Kristaps Dzonsons
Latvijas Universitaates Matemaatikas un Informaatikas Instituuts
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:
* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
* Neither the name of the copyright holders nor the names of
  contributors may be used to endorse or promote products derived from
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <sys/param.h>
#include <sys/time.h>

#include <assert.h>
#include <err.h>
#include <getopt.h>
#include <limits.h>
#include <mpi.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <SWI-Prolog.h>

#include "pl_mpi.h"


/*-[type declarations]-----------------------------------------------*/

/*-[function declarations]-------------------------------------------*/

/*-[variable definitions]--------------------------------------------*/

/*extern*/ unsigned int rank = 0;
/*extern*/ unsigned int nodes = 0;
/*extern*/ struct options opts;

/*-[function definitions]--------------------------------------------*/

static void
usage(int code)
{
	extern char *__progname;
	if (rank == 0)
		(void)fprintf(stderr, "Usage: %s "
				"[-dt] "
				"[-g goal]\n", 
				__progname);
	MPI_FINALIZE();
	exit(code);
	/* NOTREACHED */
}

int
main(int argc, char *argv[])
{ 
	extern int optind;
	int c, rc;
	char *target;

	(void)memset(&opts, 0, sizeof(struct options));

	/* Subsystem initialisations. */
	init_mpi(&argc, &argv);
	init_prolog(argc, argv);

#ifdef READLINE
	if (rank == 0)
		PL_install_readline();
#endif /*READLINE*/

	if (rank == 0)
		setproctitle("master");
	else if (rank == 1)
		setproctitle("control");
	else
		setproctitle("slave [rank %d]", rank);

	if (snprintf(opts.dump_file, MAXPATHLEN, "dump-%d.txt", 
				rank) >= MAXPATHLEN - 1)
		errx(1, "profile filename too long");
	if (snprintf(opts.trace_file, MAXPATHLEN, "trace-%u.txt",
				rank) >= MAXPATHLEN - 1)
		errx(1, "trace filename too long");

	/* Option parsing. */
	target = NULL;
	while ((c = getopt(argc, argv, "g:dt")) != -1) 
		switch (c) {
		case('d'):
			opts.dump = fopen(opts.dump_file, "w");
			if (opts.dump == NULL)
				err(1, "failed open dump: %s",
						opts.dump_file);
			break;
		case('t'):
			opts.trace = fopen(opts.trace_file, "w");
			if (opts.trace == NULL)
				err(1, "failed open trace: %s",
						opts.trace_file);
			break;
		case('g'):
			target = optarg;
			break;
		default:
			usage(1);
			/* NOTREACHED */
		}
	argc -= optind;
	argv += optind;

	MPI_BARRIER();

	/* Route into the main caller. */
	switch(rank) {
	case(0):
		if (target != NULL) {
			term_t g;
			g = PL_new_term_ref();
			if ( ! PL_chars_to_term(target, g))
				MPI_ABORT();
			rc = PL_call(g, NULL);
			PL_halt(rc);
		} else
			PL_halt(PL_toplevel());
		/* NOTREACHED */
	case(1):
		rc = loop_cntrl();
		break;
	default:
		rc = loop_slave();
		break;
	}

	return(rc);
}
