/* $Id: profile.h,v 1.1 2007-01-22 21:38:18 kristaps Exp $ */
/*
Copyright (c) 2006 Kristaps Dzonsons
Latvijas Universitaates Matemaatikas un Informaatikas Instituuts
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:
* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
* Neither the name of the copyright holders nor the names of
  contributors may be used to endorse or promote products derived from
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef PL_MPI_PROF_H
#define PL_MPI_PROF_H

__BEGIN_DECLS

/* Needed by all header files. */
#include "compat.h"

/** Maximum length of a data line. */
#define MAX_LINE		4096
/** Maximum number of input sources. */
#define MAX_SOURCES		512

#define MASTER_INPUT_FMT 	"%10p %10lu %20llu"
struct master {
	void *ctx;
	unsigned long id;
	unsigned long long wallc;
} ;

#define SLAVE_INPUT_FMT		"%10lu %7u %20llu %20llu"
struct slave {
	unsigned long id;
	unsigned long long cpu;
	unsigned long long wallc;
	unsigned int index;
};

#define CNTRL_INPUT_FMT		"%10lu %7u %7u %20llu"
struct cntrl {
	unsigned long id;
	unsigned int index;
	unsigned int slave;
	unsigned long long wallc;
};

struct io_source {
	char *file;
	int fd;
	unsigned int node;
	unsigned int line;
#define IO_SOURCE_CNTRL		0
#define IO_SOURCE_MASTER	1
#define IO_SOURCE_SLAVE		2
	unsigned int type;
#define MASTER_SCALE_SZ		256
#define CNTRL_SCALE_SZ		512
#define SLAVE_SCALE_SZ		512
	size_t scale_size;
	size_t elems;
	union {
		struct slave *slave;
		struct cntrl *cntrl;
		struct master *master;
	} data;
};

/** @return TRUE on success, FALSE on failure. */
extern int
pl_mpi_prof_init(struct io_source *src, size_t size,
		/* @return -1 on failure, 0 on success */
		int (*update)(const struct io_source *src, 
			size_t size),
		int argc, char *argv[]);

/** @param interval Check interval (seconds).  If set to 0, this will
 * read all available data and exit without polling.
 * @return TRUE on success, FALSE on failure. */
extern int
pl_mpi_prof_run(struct io_source *src, size_t size,
		unsigned int interval);

__END_DECLS

#endif /*!PL_MPI_PROF_H*/
